@@ -24,7 +24,7 @@
 
 #define CHECK(s,msg,...) if ((res = (s)) < 0) { spa_log_error(state->log, msg ": %s", ##__VA_ARGS__, snd_strerror(res)); return res; }
 
-static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_queue)
+static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_queue, bool probe_ump)
 {
 	struct props *props = &state->props;
 	int res;
@@ -37,11 +37,46 @@ static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_qu
 			   0)) < 0)
 		return res;
 
-	if ((res = snd_seq_set_client_midi_version(conn->hndl, SND_SEQ_CLIENT_UMP_MIDI_2_0)) < 0) {
-		snd_seq_close(conn->hndl);
-		spa_log_info(state->log, "%p: ALSA failed to enable UMP MIDI: %s",
-				state, snd_strerror(res));
-		return res;
+	if (!state->ump) {
+		spa_log_info(state->log, "%p: ALSA UMP MIDI disabled", state);
+		return 0;
+	}
+
+#ifdef HAVE_ALSA_UMP
+	res = snd_seq_set_client_midi_version(conn->hndl, SND_SEQ_CLIENT_UMP_MIDI_2_0);
+	if (!res) {
+		snd_seq_client_info_t *info = NULL;
+
+		/* Double check client version */
+		res = snd_seq_client_info_malloc(&info);
+		if (!res)
+			res = snd_seq_get_client_info(conn->hndl, info);
+		if (!res) {
+			res = snd_seq_client_info_get_midi_version(info);
+			if (res == SND_SEQ_CLIENT_UMP_MIDI_2_0)
+				res = 0;
+			else
+				res = -EIO;
+		}
+		if (info)
+			snd_seq_client_info_free(info);
+	}
+#else
+	res = -EOPNOTSUPP;
+#endif
+
+	if (res < 0) {
+		spa_log_lev(state->log, (probe_ump ? SPA_LOG_LEVEL_INFO : SPA_LOG_LEVEL_ERROR),
+				"%p: ALSA failed to enable UMP MIDI: %s", state, snd_strerror(res));
+		if (!probe_ump) {
+			snd_seq_close(conn->hndl);
+			return res;  /* either all are UMP or none are UMP */
+		}
+
+		state->ump = false;
+	} else {
+		spa_log_debug(state->log, "%p: ALSA UMP MIDI enabled", state);
+		state->ump = true;
 	}
 
 	return 0;
@@ -172,7 +207,7 @@ static void init_ports(struct seq_state *state)
 	}
 }
 
-static void debug_event(struct seq_state *state, snd_seq_ump_event_t *ev)
+static void debug_event(struct seq_state *state, snd_seq_event_t *ev)
 {
 	if (SPA_LIKELY(!spa_log_level_topic_enabled(state->log, SPA_LOG_TOPIC_DEFAULT, SPA_LOG_LEVEL_TRACE)))
 		return;
@@ -196,21 +231,74 @@ static void debug_event(struct seq_state *state, snd_seq_ump_event_t *ev)
 		ev->queue);
 }
 
+#ifdef HAVE_ALSA_UMP
+static void debug_ump_event(struct seq_state *state, snd_seq_ump_event_t *ev)
+{
+	if (SPA_LIKELY(!spa_log_level_topic_enabled(state->log, SPA_LOG_TOPIC_DEFAULT, SPA_LOG_LEVEL_TRACE)))
+		return;
+
+	spa_log_trace(state->log, "event type:%d flags:0x%x", ev->type, ev->flags);
+	switch (ev->flags & SND_SEQ_TIME_STAMP_MASK) {
+	case SND_SEQ_TIME_STAMP_TICK:
+		spa_log_trace(state->log, " time: %d ticks", ev->time.tick);
+		break;
+	case SND_SEQ_TIME_STAMP_REAL:
+		spa_log_trace(state->log, " time = %d.%09d",
+			(int)ev->time.time.tv_sec,
+			(int)ev->time.time.tv_nsec);
+		break;
+	}
+	spa_log_trace(state->log, " source:%d.%d dest:%d.%d queue:%d",
+		ev->source.client,
+		ev->source.port,
+		ev->dest.client,
+		ev->dest.port,
+		ev->queue);
+}
+#endif
+
 static void alsa_seq_on_sys(struct spa_source *source)
 {
 	struct seq_state *state = source->data;
-	snd_seq_ump_event_t *ev;
+	const bool ump = state->ump;
 	int res;
 
-	while (snd_seq_ump_event_input(state->sys.hndl, &ev) > 0) {
-		const snd_seq_addr_t *addr = &ev->data.addr;
+	while (1) {
+		const snd_seq_addr_t *addr;
+		snd_seq_event_type_t type;
+
+		if (ump) {
+#ifdef HAVE_ALSA_UMP
+			snd_seq_ump_event_t *ev;
+
+			res = snd_seq_ump_event_input(state->sys.hndl, &ev);
+			if (res <= 0)
+				break;
+
+			debug_ump_event(state, ev);
+
+			addr = &ev->data.addr;
+			type = ev->type;
+#else
+			spa_assert_not_reached();
+#endif
+		} else {
+			snd_seq_event_t *ev;
+
+			res = snd_seq_event_input(state->sys.hndl, &ev);
+			if (res <= 0)
+				break;
+
+			debug_event(state, ev);
+
+			addr = &ev->data.addr;
+			type = ev->type;
+		}
 
 		if (addr->client == state->event.addr.client)
 			continue;
 
-		debug_event(state, ev);
-
-		switch (ev->type) {
+		switch (type) {
 		case SND_SEQ_EVENT_CLIENT_START:
 		case SND_SEQ_EVENT_CLIENT_CHANGE:
 			spa_log_info(state->log, "client add/change %d", addr->client);
@@ -244,7 +332,7 @@ static void alsa_seq_on_sys(struct spa_source *source)
 			break;
 		default:
 			spa_log_info(state->log, "unhandled event %d: %d:%d",
-					ev->type, addr->client, addr->port);
+					type, addr->client, addr->port);
 			break;
 
 		}
@@ -269,8 +357,8 @@ int spa_alsa_seq_open(struct seq_state *state)
 
 	spa_zero(reserve);
 	for (i = 0; i < 16; i++) {
-		spa_log_debug(state->log, "close %d", i);
-		if ((res = seq_open(state, &reserve[i], false)) < 0)
+		spa_log_debug(state->log, "open %d", i);
+		if ((res = seq_open(state, &reserve[i], false, (i == 0))) < 0)
 			break;
 	}
 	if (i >= 2) {
@@ -529,21 +617,58 @@ static int process_recycle(struct seq_state *state)
 
 static int process_read(struct seq_state *state)
 {
-	snd_seq_ump_event_t *ev;
 	struct seq_stream *stream = &state->streams[SPA_DIRECTION_OUTPUT];
+	const bool ump = state->ump;
 	uint32_t i;
 	uint32_t *data;
+	uint8_t midi1_data[MAX_EVENT_SIZE];
+	uint32_t ump_data[MAX_EVENT_SIZE];
 	long size;
-	int res;
+	int res = -1;
 
 	/* copy all new midi events into their port buffers */
-	while ((res = snd_seq_ump_event_input(state->event.hndl, &ev)) > 0) {
-		const snd_seq_addr_t *addr = &ev->source;
+	while (1) {
+		const snd_seq_addr_t *addr;
 		struct seq_port *port;
 		uint64_t ev_time, diff;
 		uint32_t offset;
+		void *event;
+		uint8_t *midi1_ptr;
+		size_t midi1_size = 0;
+		uint64_t ump_state = 0;
+		snd_seq_event_type_t SPA_UNUSED type;
 
-		debug_event(state, ev);
+		if (ump) {
+#ifdef HAVE_ALSA_UMP
+			snd_seq_ump_event_t *ev;
+
+			res = snd_seq_ump_event_input(state->event.hndl, &ev);
+			if (res <= 0)
+				break;
+
+			debug_ump_event(state, ev);
+
+			event = ev;
+			addr = &ev->source;
+			ev_time = SPA_TIMESPEC_TO_NSEC(&ev->time.time);
+			type = ev->type;
+#else
+			spa_assert_not_reached();
+#endif
+		} else {
+			snd_seq_event_t *ev;
+
+			res = snd_seq_event_input(state->event.hndl, &ev);
+			if (res <= 0)
+				break;
+
+			debug_event(state, ev);
+
+			event = ev;
+			addr = &ev->source;
+			ev_time = SPA_TIMESPEC_TO_NSEC(&ev->time.time);
+			type = ev->type;
+		}
 
 		if ((port = find_port(state, stream, addr)) == NULL) {
 			spa_log_debug(state->log, "unknown port %d.%d",
@@ -559,12 +684,8 @@ static int process_read(struct seq_state *state)
 			continue;
 		}
 
-		data = (uint32_t*)&ev->ump[0];
-		size = spa_ump_message_size(snd_ump_msg_hdr_type(ev->ump[0])) * 4;
-
 		/* queue_time is the estimated current time of the queue as calculated by
 		 * the DLL. Calculate the age of the event. */
-		ev_time = SPA_TIMESPEC_TO_NSEC(&ev->time.time);
 		if (state->queue_time > ev_time)
 			diff = state->queue_time - ev_time;
 		else
@@ -577,19 +698,54 @@ static int process_read(struct seq_state *state)
 		else
 			offset = 0;
 
-		spa_log_trace_fp(state->log, "event %d time:%"PRIu64" offset:%d size:%ld port:%d.%d",
-				ev->type, ev_time, offset, size, addr->client, addr->port);
+		if (ump) {
+#ifdef HAVE_ALSA_UMP
+			snd_seq_ump_event_t *ev = event;
 
-		spa_pod_builder_control(&port->builder, offset, SPA_CONTROL_UMP);
-		spa_pod_builder_bytes(&port->builder, data, size);
+			data = (uint32_t*)&ev->ump[0];
+			size = spa_ump_message_size(snd_ump_msg_hdr_type(ev->ump[0])) * 4;
+#else
+			spa_assert_not_reached();
+#endif
+		} else {
+			snd_seq_event_t *ev = event;
 
-		/* make sure we can fit at least one control event of max size otherwise
-		 * we keep the event in the queue and try to copy it in the next cycle */
-		if (port->builder.state.offset +
-		    sizeof(struct spa_pod_control) +
-		    MAX_EVENT_SIZE > port->buffer->buf->datas[0].maxsize)
-			break;
+			snd_midi_event_reset_decode(stream->codec);
+			if ((size = snd_midi_event_decode(stream->codec, midi1_data, sizeof(midi1_data), ev)) < 0) {
+				spa_log_warn(state->log, "decode failed: %s", snd_strerror(size));
+				continue;
+			}
+
+			midi1_ptr = midi1_data;
+			midi1_size = size;
+		}
+
+		do {
+			if (!ump) {
+				data = ump_data;
+				size = spa_ump_from_midi(&midi1_ptr, &midi1_size,
+						ump_data, sizeof(ump_data), 0, &ump_state);
+				if (size <= 0)
+					break;
+			}
+
+			spa_log_trace_fp(state->log, "event %d time:%"PRIu64" offset:%d size:%ld port:%d.%d",
+					type, ev_time, offset, size, addr->client, addr->port);
+
+			spa_pod_builder_control(&port->builder, offset, SPA_CONTROL_UMP);
+			spa_pod_builder_bytes(&port->builder, data, size);
+
+			/* make sure we can fit at least one control event of max size otherwise
+			 * we keep the event in the queue and try to copy it in the next cycle */
+			if (port->builder.state.offset +
+					sizeof(struct spa_pod_control) +
+					MAX_EVENT_SIZE > port->buffer->buf->datas[0].maxsize)
+				goto done;
+
+		} while (!ump);
         }
+
+done:
 	if (res < 0 && res != -EAGAIN)
 		spa_log_warn(state->log, "event read failed: %s", snd_strerror(res));
 
@@ -651,6 +807,7 @@ static int process_read(struct seq_state *state)
 static int process_write(struct seq_state *state)
 {
 	struct seq_stream *stream = &state->streams[SPA_DIRECTION_INPUT];
+	const bool ump = state->ump;
 	uint32_t i;
 	int err, res = 0;
 
@@ -661,7 +818,6 @@ static int process_write(struct seq_state *state)
 		struct spa_pod_sequence *pod;
 		struct spa_data *d;
 		struct spa_pod_control *c;
-		snd_seq_ump_event_t ev;
 		uint64_t out_time;
 		snd_seq_real_time_t out_rt;
 
@@ -695,25 +851,55 @@ static int process_write(struct seq_state *state)
 
 			body = SPA_POD_BODY(&c->value);
 			body_size = SPA_POD_BODY_SIZE(&c->value);
-			spa_zero(ev);
-
-			memcpy(ev.ump, body, SPA_MIN(sizeof(ev.ump), (size_t)body_size));
-
-			snd_seq_ev_set_source(&ev, state->event.addr.port);
-			snd_seq_ev_set_dest(&ev, port->addr.client, port->addr.port);
 
 			out_time = state->queue_time + NSEC_FROM_CLOCK(&state->rate, c->offset);
-
 			out_rt.tv_nsec = out_time % SPA_NSEC_PER_SEC;
 			out_rt.tv_sec = out_time / SPA_NSEC_PER_SEC;
-			snd_seq_ev_schedule_real(&ev, state->event.queue_id, 0, &out_rt);
 
-			spa_log_trace_fp(state->log, "event %d time:%"PRIu64" offset:%d size:%zd port:%d.%d",
-				ev.type, out_time, c->offset, body_size, port->addr.client, port->addr.port);
+			spa_log_trace_fp(state->log, "event time:%"PRIu64" offset:%d size:%zd port:%d.%d",
+					out_time, c->offset, body_size, port->addr.client, port->addr.port);
 
-			if ((err = snd_seq_ump_event_output(state->event.hndl, &ev)) < 0) {
-				spa_log_warn(state->log, "failed to output event: %s",
-						snd_strerror(err));
+			if (ump) {
+#ifdef HAVE_ALSA_UMP
+				snd_seq_ump_event_t ev;
+
+				snd_seq_ump_ev_clear(&ev);
+				snd_seq_ev_set_ump_data(&ev, body, SPA_MIN(sizeof(ev.ump), (size_t)body_size));
+				snd_seq_ev_set_source(&ev, state->event.addr.port);
+				snd_seq_ev_set_dest(&ev, port->addr.client, port->addr.port);
+				snd_seq_ev_schedule_real(&ev, state->event.queue_id, 0, &out_rt);
+
+				if ((err = snd_seq_ump_event_output(state->event.hndl, &ev)) < 0) {
+					spa_log_warn(state->log, "failed to output event: %s",
+							snd_strerror(err));
+				}
+#else
+				spa_assert_not_reached();
+#endif
+			} else {
+				snd_seq_event_t ev;
+				uint8_t data[MAX_EVENT_SIZE];
+				int size;
+
+				if ((size = spa_ump_to_midi((uint32_t *)body, body_size, data, sizeof(data))) <= 0)
+					continue;
+
+				snd_seq_ev_clear(&ev);
+
+				snd_midi_event_reset_encode(stream->codec);
+				if ((size = snd_midi_event_encode(stream->codec, data, size, &ev)) <= 0) {
+					spa_log_warn(state->log, "failed to encode event: %s", snd_strerror(size));
+					continue;
+				}
+
+				snd_seq_ev_set_source(&ev, state->event.addr.port);
+				snd_seq_ev_set_dest(&ev, port->addr.client, port->addr.port);
+				snd_seq_ev_schedule_real(&ev, state->event.queue_id, 0, &out_rt);
+
+				if ((err = snd_seq_event_output(state->event.hndl, &ev)) < 0) {
+					spa_log_warn(state->log, "failed to output event: %s",
+							snd_strerror(err));
+				}
 			}
 		}
 	}
