@@ -997,6 +997,8 @@ found:
 	format = merge_objects(this, &b, SPA_PARAM_Format,
 			(struct spa_pod_object*)format,
 			(struct spa_pod_object*)def);
+	if (format == NULL)
+		return -ENOSPC;
 
 	spa_pod_fixate(format);
 
@@ -1612,13 +1614,13 @@ port_enum_formats_for_convert(struct impl *this, int seq, enum spa_direction dir
 	uint32_t count = 0;
 	struct spa_result_node_params result;
 
-	spa_pod_builder_init(&b, buffer, sizeof(buffer));
-
 	result.id = id;
 	result.next = start;
 next:
 	result.index = result.next;
 
+	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+
 	if (result.next < 0x100000) {
 		/* Enumerate follower formats first, until we have enough or we run out */
 		if ((res = node_port_enum_params_sync(this, this->follower, direction, port_id, id,
