@@ -9,11 +9,14 @@
 extern "C" {
 #endif
 
+#include <stdio.h>
+
 #include <spa/utils/defs.h>
 #include <spa/pod/pod.h>
 
 #include <pipewire/pipewire.h>
 
+struct client;
 struct pw_manager_object;
 
 struct pw_manager_events {
@@ -68,8 +71,8 @@ struct pw_manager_object {
 	struct pw_properties *props;
 	struct pw_proxy *proxy;
 	char *message_object_path;
-	int (*message_handler)(struct pw_manager *m, struct pw_manager_object *o,
-	                       const char *message, const char *params, char **response);
+	int (*message_handler)(struct client *client, struct pw_manager_object *o,
+	                       const char *message, const char *params, FILE *response);
 
 	void *info;
 	struct spa_param_info *params;
