@@ -16,7 +16,7 @@ static void rtp_opus_process_playback(void *data)
 	int32_t avail;
 
 	if ((buf = pw_stream_dequeue_buffer(impl->stream)) == NULL) {
-		pw_log_debug("Out of stream buffers: %m");
+		pw_log_info("Out of stream buffers: %m");
 		return;
 	}
 	d = buf->buffer->datas;
@@ -49,7 +49,7 @@ static void rtp_opus_process_playback(void *data)
 		pw_log(level, "underrun %d/%u < %u",
 					avail, target_buffer, wanted);
 	} else {
-		float error, corr;
+		double error, corr;
 		if (impl->first) {
 			if ((uint32_t)avail > target_buffer) {
 				uint32_t skip = avail - target_buffer;
@@ -68,19 +68,15 @@ static void rtp_opus_process_playback(void *data)
 			/* when not using direct timestamp and clocks are not
 			 * in sync, try to adjust our playback rate to keep the
 			 * requested target_buffer bytes in the ringbuffer */
-			error = (float)target_buffer - (float)avail;
-			error = SPA_CLAMP(error, -impl->max_error, impl->max_error);
+			error = (double)target_buffer - (double)avail;
+			error = SPA_CLAMPD(error, -impl->max_error, impl->max_error);
 
 			corr = spa_dll_update(&impl->dll, error);
 
-			pw_log_debug("avail:%u target:%u error:%f corr:%f", avail,
+			pw_log_trace("avail:%u target:%u error:%f corr:%f", avail,
 					target_buffer, error, corr);
 
-			if (impl->io_rate_match) {
-				SPA_FLAG_SET(impl->io_rate_match->flags,
-						SPA_IO_RATE_MATCH_FLAG_ACTIVE);
-				impl->io_rate_match->rate = 1.0f / corr;
-			}
+			pw_stream_set_rate(impl->stream, 1.0 / corr);
 		}
 		spa_ringbuffer_read_data(&impl->ring,
 				impl->buffer,
@@ -183,7 +179,7 @@ static int rtp_opus_receive(struct impl *impl, uint8_t *buffer, ssize_t len)
 		if (end > BUFFER_SIZE2)
 			memmove(impl->buffer, &impl->buffer[BUFFER_SIZE2], end - BUFFER_SIZE2);
 
-		pw_log_debug("receiving %zd len:%d timestamp:%d %u", plen, res, timestamp, index);
+		pw_log_info("receiving %zd len:%d timestamp:%d %u", plen, res, timestamp, index);
 		samples = res;
 
 		write += samples;
@@ -196,14 +192,18 @@ short_packet:
 	return -EINVAL;
 invalid_version:
 	pw_log_warn("invalid RTP version");
-	spa_debug_mem(0, buffer, len);
+	spa_debug_log_mem(pw_log_get(), SPA_LOG_LEVEL_INFO, 0, buffer, len);
 	return -EPROTO;
 invalid_len:
 	pw_log_warn("invalid RTP length");
 	return -EINVAL;
 unexpected_ssrc:
-	pw_log_warn("unexpected SSRC (expected %u != %u)",
-		impl->ssrc, hdr->ssrc);
+	if (!impl->fixed_ssrc) {
+		/* We didn't have a configured SSRC, and there's more than one SSRC on
+		* this address/port pair */
+		pw_log_warn("unexpected SSRC (expected %u != %u)",
+			impl->ssrc, hdr->ssrc);
+	}
 	return -EINVAL;
 }
 
@@ -244,7 +244,7 @@ static void rtp_opus_flush_packets(struct impl *impl)
 				(const float*)&impl->buffer[offset * stride], tosend,
 				out, sizeof(out));
 
-		pw_log_debug("sending %d len:%d timestamp:%d", tosend, res, timestamp);
+		pw_log_trace("sending %d len:%d timestamp:%d", tosend, res, timestamp);
 		iov[1].iov_len = res;
 
 		rtp_stream_emit_send_packet(impl, iov, 2);
@@ -255,7 +255,7 @@ static void rtp_opus_flush_packets(struct impl *impl)
 		avail -= tosend;
 	}
 
-	pw_log_debug("move %d offset:%d", avail, offset);
+	pw_log_trace("move %d offset:%d", avail, offset);
 	memmove(impl->buffer, &impl->buffer[offset * stride], avail * stride);
 
 	spa_ringbuffer_read_update(&impl->ring, timestamp);
@@ -270,7 +270,7 @@ static void rtp_opus_process_capture(void *data)
 	int32_t filled, wanted;
 
 	if ((buf = pw_stream_dequeue_buffer(impl->stream)) == NULL) {
-		pw_log_debug("Out of stream buffers: %m");
+		pw_log_info("Out of stream buffers: %m");
 		return;
 	}
 	d = buf->buffer->datas;
