@@ -30,14 +30,14 @@ static void vban_audio_process_playback(void *data)
 		memset(d[0].data, 0, wanted * stride);
 		if (impl->have_sync) {
 			impl->have_sync = false;
-			level = SPA_LOG_LEVEL_WARN;
+			level = SPA_LOG_LEVEL_INFO;
 		} else {
 			level = SPA_LOG_LEVEL_DEBUG;
 		}
 		pw_log(level, "underrun %d/%u < %u",
 					avail, target_buffer, wanted);
 	} else {
-		float error, corr;
+		double error, corr;
 		if (impl->first) {
 			if ((uint32_t)avail > target_buffer) {
 				uint32_t skip = avail - target_buffer;
@@ -54,19 +54,16 @@ static void vban_audio_process_playback(void *data)
 		}
 		/* try to adjust our playback rate to keep the
 		 * requested target_buffer bytes in the ringbuffer */
-		error = (float)target_buffer - (float)avail;
-		error = SPA_CLAMP(error, -impl->max_error, impl->max_error);
+		error = (double)target_buffer - (double)avail;
+		error = SPA_CLAMPD(error, -impl->max_error, impl->max_error);
 
-		corr = (float)spa_dll_update(&impl->dll, error);
+		corr = spa_dll_update(&impl->dll, error);
 
 		pw_log_debug("avail:%u target:%u error:%f corr:%f", avail,
 				target_buffer, error, corr);
 
-		if (impl->io_rate_match) {
-			SPA_FLAG_SET(impl->io_rate_match->flags,
-					SPA_IO_RATE_MATCH_FLAG_ACTIVE);
-			impl->io_rate_match->rate = 1.0f / corr;
-		}
+		pw_stream_set_rate(impl->stream, 1.0 / corr);
+
 		spa_ringbuffer_read_data(&impl->ring,
 				impl->buffer,
 				BUFFER_SIZE,
@@ -92,12 +89,7 @@ static int vban_audio_receive(struct impl *impl, uint8_t *buffer, ssize_t len)
 	uint32_t stride = impl->stride;
 	int32_t filled;
 
-	if (len < VBAN_HEADER_SIZE)
-		goto short_packet;
-
 	hdr = (struct vban_header*)buffer;
-	if (strncmp(hdr->vban, "VBAN", 3))
-		goto invalid_version;
 
 	impl->receiving = true;
 
@@ -155,14 +147,6 @@ static int vban_audio_receive(struct impl *impl, uint8_t *buffer, ssize_t len)
 		spa_ringbuffer_write_update(&impl->ring, write);
 	}
 	return 0;
-
-short_packet:
-	pw_log_warn("short packet received");
-	return -EINVAL;
-invalid_version:
-	pw_log_warn("invalid VBAN version");
-	spa_debug_log_mem(pw_log_get(), SPA_LOG_LEVEL_INFO, 0, buffer, len);
-	return -EPROTO;
 }
 
 static inline void
