@@ -146,6 +146,7 @@ struct filter {
 	unsigned int warn_mlock:1;
 	unsigned int trigger:1;
 	int in_emit_param_changed;
+	int pending_drain;
 };
 
 static int get_param_index(uint32_t id)
@@ -406,8 +407,10 @@ static bool filter_set_state(struct pw_filter *filter, enum pw_filter_state stat
 			     pw_filter_state_as_string(old),
 			     pw_filter_state_as_string(state), res, error);
 
-		if (state == PW_FILTER_STATE_ERROR)
+		if (state == PW_FILTER_STATE_ERROR) {
 			pw_log_error("%p: error (%d) %s", filter, res, error);
+			errno = -res;
+		}
 
 		filter->state = state;
 		pw_filter_emit_state_changed(filter, old, state, error);
@@ -989,13 +992,19 @@ do_call_drained(struct spa_loop *loop,
 	struct pw_filter *filter = &impl->this;
 	pw_log_trace("%p: drained", filter);
 	pw_filter_emit_drained(filter);
+	SPA_ATOMIC_DEC(impl->pending_drain);
 	return 0;
 }
 
 static void call_drained(struct filter *impl)
 {
-	pw_loop_invoke(impl->main_loop,
-		do_call_drained, 1, NULL, 0, false, impl);
+	pw_log_info("%p: drained", impl);
+	if (SPA_ATOMIC_INC(impl->pending_drain) == 1) {
+		pw_loop_invoke(impl->main_loop,
+			do_call_drained, 1, NULL, 0, false, impl);
+	} else {
+		SPA_ATOMIC_DEC(impl->pending_drain);
+	}
 }
 
 static int impl_node_process(void *object)
@@ -1119,11 +1128,14 @@ static void proxy_destroy(void *_data)
 static void proxy_error(void *_data, int seq, int res, const char *message)
 {
 	struct pw_filter *filter = _data;
+	int old_errno = errno;
 	/* we just emit the state change here to inform the application.
 	 * If this is supposed to be a permanent error, the app should
 	 * do a pw_filter_set_error() */
+	errno = -res;
 	pw_filter_emit_state_changed(filter, filter->state,
 			PW_FILTER_STATE_ERROR, message);
+	errno = old_errno;
 }
 
 static void proxy_bound_props(void *_data, uint32_t global_id, const struct spa_dict *props)
@@ -1467,6 +1479,8 @@ enum pw_filter_state pw_filter_get_state(struct pw_filter *filter, const char **
 {
 	if (error)
 		*error = filter->error;
+	if (filter->state == PW_FILTER_STATE_ERROR)
+		errno = -filter->error_res;
 	return filter->state;
 }
 
@@ -1762,17 +1776,26 @@ static void add_video_dsp_port_params(struct filter *impl, struct port *port)
 			SPA_FORMAT_VIDEO_format,   SPA_POD_Id(SPA_VIDEO_FORMAT_DSP_F32)));
 }
 
-static void add_control_dsp_port_params(struct filter *impl, struct port *port)
+static void add_control_dsp_port_params(struct filter *impl, struct port *port, uint32_t types)
 {
 	uint8_t buffer[4096];
 	struct spa_pod_builder b;
+	struct spa_pod_frame f[1];
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+	spa_pod_builder_push_object(&b, &f[0],
+		SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
+	spa_pod_builder_add(&b,
+		SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_application),
+		SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_control),
+		0);
+	if (types != 0) {
+		spa_pod_builder_add(&b,
+			SPA_FORMAT_CONTROL_types, SPA_POD_CHOICE_FLAGS_Int(types),
+			0);
+	}
 	add_param(impl, port, SPA_PARAM_EnumFormat, PARAM_FLAG_LOCKED,
-		spa_pod_builder_add_object(&b,
-			SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
-			SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_application),
-			SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_control)));
+			spa_pod_builder_pop(&b, &f[0]));
 }
 
 SPA_EXPORT
@@ -1828,9 +1851,12 @@ void *pw_filter_add_port(struct pw_filter *filter,
 			add_audio_dsp_port_params(impl, p);
 		else if (spa_streq(str, "32 bit float RGBA video"))
 			add_video_dsp_port_params(impl, p);
-		else if (spa_streq(str, "8 bit raw midi") ||
-		    spa_streq(str, "8 bit raw control"))
-			add_control_dsp_port_params(impl, p);
+		else if (spa_streq(str, "8 bit raw midi"))
+			add_control_dsp_port_params(impl, p, 1u << SPA_CONTROL_Midi);
+		else if (spa_streq(str, "8 bit raw control"))
+			add_control_dsp_port_params(impl, p, 0);
+		else if (spa_streq(str, "32 bit raw UMP"))
+			add_control_dsp_port_params(impl, p, 1u << SPA_CONTROL_UMP);
 	}
 	/* then override with user provided if any */
 	if (update_params(impl, p, SPA_ID_INVALID, params, n_params) < 0)
@@ -2079,20 +2105,23 @@ do_trigger_process(struct spa_loop *loop,
 	return spa_node_call_ready(&impl->callbacks, res);
 }
 
-static int do_trigger_request_process(struct spa_loop *loop,
+static int do_emit_event(struct spa_loop *loop,
                  bool async, uint32_t seq, const void *data, size_t size, void *user_data)
 {
 	struct filter *impl = user_data;
-	uint8_t buffer[1024];
-	struct spa_pod_builder b = { 0 };
-
-	spa_pod_builder_init(&b, buffer, sizeof(buffer));
-	spa_node_emit_event(&impl->hooks,
-			spa_pod_builder_add_object(&b,
-				SPA_TYPE_EVENT_Node, SPA_NODE_EVENT_RequestProcess));
+	const struct spa_event *event = data;
+	spa_node_emit_event(&impl->hooks, event);
 	return 0;
 }
 
+SPA_EXPORT
+int pw_filter_emit_event(struct pw_filter *filter, const struct spa_event *event)
+{
+	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+	return pw_loop_invoke(impl->main_loop,
+		do_emit_event, 1, event, SPA_POD_SIZE(&event->pod), false, impl);
+}
+
 SPA_EXPORT
 int pw_filter_trigger_process(struct pw_filter *filter)
 {
@@ -2102,13 +2131,13 @@ int pw_filter_trigger_process(struct pw_filter *filter)
 	pw_log_trace_fp("%p: driving:%d", impl, filter->node->driving);
 
 	if (impl->trigger) {
-		pw_impl_node_trigger(filter->node);
+		res = pw_impl_node_trigger(filter->node);
 	} else if (filter->node->driving) {
 		res = pw_loop_invoke(impl->data_loop,
 			do_trigger_process, 1, NULL, 0, false, impl);
 	} else {
-		res = pw_loop_invoke(impl->main_loop,
-			do_trigger_request_process, 1, NULL, 0, false, impl);
+		pw_filter_emit_event(filter,
+				&SPA_NODE_EVENT_INIT(SPA_NODE_EVENT_RequestProcess));
 	}
 	return res;
 }
