@@ -22,6 +22,7 @@
 #include <spa/utils/string.h>
 #include <spa/utils/json-pod.h>
 
+#define PW_API_NODE_IMPL	SPA_EXPORT
 #include "pipewire/impl-node.h"
 #include "pipewire/private.h"
 
@@ -47,6 +48,8 @@ struct impl {
 	unsigned int cache_params:1;
 	unsigned int pending_play:1;
 
+	struct spa_command *pending_request_process;
+
 	char *group;
 	char *link_group;
 	char *sync_group;
@@ -334,7 +337,7 @@ static int start_node(struct pw_impl_node *this)
 	} else {
 		/* driver nodes will wait until all other nodes are started before
 		 * they are started */
-		this->pending_request_process = 0;
+		spa_clear_ptr(impl->pending_request_process, free);
 		res = EBUSY;
 	}
 
@@ -439,7 +442,7 @@ static void node_update_state(struct pw_impl_node *node, enum pw_node_state stat
 				state = PW_NODE_STATE_ERROR;
 				error = spa_aprintf("Start error: %s", spa_strerror(res));
 				remove_node_from_graph(node);
-			} else if (node->pending_request_process > 0) {
+			} else if (impl->pending_request_process != NULL) {
 				emit_pending_request_process = true;
 			}
 		}
@@ -448,7 +451,8 @@ static void node_update_state(struct pw_impl_node *node, enum pw_node_state stat
 	case PW_NODE_STATE_SUSPENDED:
 	case PW_NODE_STATE_ERROR:
 		if (state != PW_NODE_STATE_IDLE || node->pause_on_idle)
-			remove_node_from_graph(node);
+			if (old != PW_NODE_STATE_CREATING)
+				remove_node_from_graph(node);
 		break;
 	default:
 		break;
@@ -475,10 +479,9 @@ static void node_update_state(struct pw_impl_node *node, enum pw_node_state stat
 	pw_impl_node_emit_state_changed(node, old, state, error);
 
 	if (emit_pending_request_process) {
-		pw_log_debug("%p: request process:%d", node, node->pending_request_process);
-		node->pending_request_process = 0;
-		spa_node_send_command(node->node,
-			    &SPA_NODE_COMMAND_INIT(SPA_NODE_COMMAND_RequestProcess));
+		pw_log_debug("%p: request process:%p", node, impl->pending_request_process);
+		spa_node_send_command(node->node, impl->pending_request_process);
+		spa_clear_ptr(impl->pending_request_process, free);
 	}
 
 	node->info.change_mask |= PW_NODE_CHANGE_MASK_STATE;
@@ -834,7 +837,9 @@ int pw_impl_node_set_io(struct pw_impl_node *this, uint32_t id, void *data, size
 
 	res = spa_node_set_io(this->node, id, data, size);
 
-	if (res >= 0 && !SPA_RESULT_IS_ASYNC(res) && this->rt.position)
+	if (this->rt.position &&
+	    ((res >= 0 && !SPA_RESULT_IS_ASYNC(res)) ||
+	    this->rt.target.activation->client_version < 1))
 		this->rt.target.activation->active_driver_id = this->rt.position->clock.id;
 
 	pw_log_debug("%p: set io: %s", this, spa_strerror(res));
@@ -1114,23 +1119,21 @@ static void check_properties(struct pw_impl_node *node)
 	const char *str, *recalc_reason = NULL;
 	struct spa_fraction frac;
 	uint32_t value;
-	bool driver, trigger, transport, sync, async;
+	bool driver, trigger, sync, async;
 	struct match match;
 
 	match = MATCH_INIT(node);
 	pw_context_conf_section_match_rules(context, "node.rules",
 			&node->properties->dict, execute_match, &match);
 
-	if ((str = pw_properties_get(node->properties, PW_KEY_PRIORITY_DRIVER))) {
-		value = pw_properties_parse_int(str);
-		if (value != node->priority_driver) {
-			pw_log_debug("%p: priority driver %d -> %d", node, node->priority_driver, value);
-			node->priority_driver = value;
-			if (node->registered && node->driver) {
-				remove_driver(context, node);
-				insert_driver(context, node);
-				recalc_reason = "driver priority changed";
-			}
+	value = pw_properties_get_uint32(node->properties, PW_KEY_PRIORITY_DRIVER, 0);
+	if (value != node->priority_driver) {
+		pw_log_debug("%p: priority driver %d -> %d", node, node->priority_driver, value);
+		node->priority_driver = value;
+		if (node->registered && node->driver) {
+			remove_driver(context, node);
+			insert_driver(context, node);
+			recalc_reason = "driver priority changed";
 		}
 	}
 	node->supports_lazy = pw_properties_get_uint32(node->properties, PW_KEY_NODE_SUPPORTS_LAZY, 0);
@@ -1219,10 +1222,13 @@ static void check_properties(struct pw_impl_node *node)
 		recalc_reason = "sync changed";
 	}
 
-	transport = pw_properties_get_bool(node->properties, PW_KEY_NODE_TRANSPORT, false);
-	if (transport != node->transport) {
-		pw_log_info("%p: transport %d -> %d", node, node->transport, transport);
-		node->transport = transport;
+	str = pw_properties_get(node->properties, PW_KEY_NODE_TRANSPORT);
+	if (str != NULL) {
+		node->transport = spa_atob(str) ?
+			PW_NODE_ACTIVATION_COMMAND_START :
+			PW_NODE_ACTIVATION_COMMAND_STOP;
+		pw_log_info("%p: transport %d", node, node->transport);
+		pw_properties_set(node->properties, PW_KEY_NODE_TRANSPORT, NULL);
 		recalc_reason = "transport changed";
 	}
 	async = pw_properties_get_bool(node->properties, PW_KEY_NODE_ASYNC, false);
@@ -1230,6 +1236,7 @@ static void check_properties(struct pw_impl_node *node)
 	if (async != node->async) {
 		pw_log_info("%p: async %d -> %d", node, node->async, async);
 		node->async = async;
+		SPA_FLAG_UPDATE(node->rt.target.activation->flags, PW_NODE_ACTIVATION_FLAG_ASYNC, async);
 	}
 
 	if ((str = pw_properties_get(node->properties, PW_KEY_MEDIA_CLASS)) != NULL &&
@@ -1277,13 +1284,11 @@ static void check_properties(struct pw_impl_node *node)
 	}
 	node->lock_quantum = pw_properties_get_bool(node->properties, PW_KEY_NODE_LOCK_QUANTUM, false);
 
-	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_FORCE_QUANTUM))) {
-		if (spa_atou32(str, &value, 0) &&
-		    node->force_quantum != value) {
-		        node->force_quantum = value;
-			node->stamp = ++context->stamp;
-			recalc_reason = "force quantum changed";
-		}
+	value = pw_properties_get_uint32(node->properties, PW_KEY_NODE_FORCE_QUANTUM, 0);
+	if (node->force_quantum != value) {
+	        node->force_quantum = value;
+		node->stamp = ++context->stamp;
+		recalc_reason = "force quantum changed";
 	}
 
 	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_RATE))) {
@@ -1299,18 +1304,17 @@ static void check_properties(struct pw_impl_node *node)
 	}
 	node->lock_rate = pw_properties_get_bool(node->properties, PW_KEY_NODE_LOCK_RATE, false);
 
-	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_FORCE_RATE))) {
-		if (spa_atou32(str, &value, 0)) {
-			if (value == 0)
-				value = node->rate.denom;
-			if (node->force_rate != value) {
-				pw_log_info("(%s-%u) force-rate:%u -> %u", node->name,
-							node->info.id, node->force_rate, value);
-				node->force_rate = value;
-				node->stamp = ++context->stamp;
-				recalc_reason = "force rate changed";
-			}
-		}
+	value = pw_properties_get_uint32(node->properties, PW_KEY_NODE_FORCE_RATE, SPA_ID_INVALID);
+	if (value == 0)
+		value = node->rate.denom;
+	if (value == SPA_ID_INVALID)
+		value = 0;
+	if (node->force_rate != value) {
+		pw_log_info("(%s-%u) force-rate:%u -> %u", node->name,
+					node->info.id, node->force_rate, value);
+		node->force_rate = value;
+		node->stamp = ++context->stamp;
+		recalc_reason = "force rate changed";
 	}
 
 	pw_log_debug("%p: driver:%d recalc:%s active:%d", node, node->driver,
@@ -1354,7 +1358,7 @@ static inline void debug_xrun_target(struct pw_impl_node *driver,
 	enum spa_log_level level = SPA_LOG_LEVEL_DEBUG;
 
 	if ((suppressed = spa_ratelimit_test(&driver->rt.rate_limit, nsec)) >= 0)
-		level = SPA_LOG_LEVEL_WARN;
+		level = SPA_LOG_LEVEL_INFO;
 
 	pw_log(level, "(%s-%u) xrun state:%p pending:%d/%d s:%"PRIu64" a:%"PRIu64" f:%"PRIu64
 		" waiting:%"PRIu64" process:%"PRIu64" status:%s (%d suppressed)",
@@ -1375,7 +1379,7 @@ static inline void debug_xrun_graph(struct pw_impl_node *driver, uint64_t nsec,
 	struct pw_node_target *t;
 
 	if ((suppressed = spa_ratelimit_test(&driver->rt.rate_limit, nsec)) >= 0)
-		level = SPA_LOG_LEVEL_WARN;
+		level = SPA_LOG_LEVEL_INFO;
 
 	pw_log(level, "(%s-%u) graph xrun %s (%d suppressed)",
 			driver->name, driver->info.id, str_status(old_status), suppressed);
@@ -1409,7 +1413,7 @@ static void debug_sync_timeout(struct pw_impl_node *driver, uint64_t nsec)
 	int suppressed;
 
 	if ((suppressed = spa_ratelimit_test(&driver->rt.rate_limit, nsec)) >= 0)
-		level = SPA_LOG_LEVEL_WARN;
+		level = SPA_LOG_LEVEL_INFO;
 
 	pw_log(level, "(%s-%u) sync timeout, going to RUNNING (%d suppressed)",
 				driver->name, driver->info.id, suppressed);
@@ -1537,8 +1541,7 @@ int pw_impl_node_trigger(struct pw_impl_node *node)
 {
 	uint64_t nsec = get_time_ns(node->rt.target.system);
 	struct pw_node_target *t = &node->rt.target;
-	t->trigger(t, nsec);
-	return 0;
+	return t->trigger(t, nsec);
 }
 
 static void node_on_fd_events(struct spa_source *source)
@@ -1901,16 +1904,16 @@ static void node_result(void *data, int seq, int res, uint32_t type, const void
 	pw_impl_node_emit_result(node, seq, res, type, result);
 }
 
-static void handle_request_process(struct pw_impl_node *node)
+static void handle_request_process_command(struct pw_impl_node *node, const struct spa_command *command)
 {
 	struct impl *impl = SPA_CONTAINER_OF(node, struct impl, this);
 	if (node->driving) {
 		pw_log_debug("request process %d %d", node->info.state, impl->pending_state);
 		if (node->info.state == PW_NODE_STATE_RUNNING) {
-			spa_node_send_command(node->driver_node->node,
-				    &SPA_NODE_COMMAND_INIT(SPA_NODE_COMMAND_RequestProcess));
+			spa_node_send_command(node->driver_node->node, command);
 		} else if (impl->pending_state == PW_NODE_STATE_RUNNING) {
-			node->pending_request_process++;
+			spa_clear_ptr(impl->pending_request_process, free);
+			impl->pending_request_process = (struct spa_command*)spa_pod_copy(&command->pod);
 		}
 	}
 }
@@ -1932,9 +1935,18 @@ static void node_event(void *data, const struct spa_event *event)
 		break;
 	case SPA_NODE_EVENT_RequestProcess:
 		if (!node->driving && !node->exported) {
+			struct spa_command *command;
+			size_t size = SPA_POD_SIZE(&event->pod);
+
+			/* turn the event and all the arguments into a command */
+			command = alloca(size);
+			memcpy(command, event, size);
+			command->body.body.type = SPA_TYPE_COMMAND_Node;
+			command->body.body.id = SPA_NODE_COMMAND_RequestProcess;
+
 			/* send the request process to the driver but only on the
 			 * server size */
-			handle_request_process(node->driver_node);
+			handle_request_process_command(node->driver_node, command);
 		}
 		break;
 	default:
@@ -2048,12 +2060,13 @@ static int node_ready(void *data, int status)
 	struct pw_impl_node *node = data;
 	struct pw_impl_node *driver = node->driver_node;
 	struct pw_node_activation *a = node->rt.target.activation;
+	struct pw_node_activation_state *state = &a->state[0];
 	struct spa_system *data_system = node->rt.target.system;
 	struct pw_node_target *t, *reposition_target = NULL;;
 	struct pw_impl_port *p;
 	struct spa_io_clock *cl = &node->rt.position->clock;
 	int sync_type, all_ready, update_sync, target_sync, old_status;
-	uint32_t owner[2], reposition_owner;
+	uint32_t owner[2], reposition_owner, pending;
 	uint64_t min_timeout = UINT64_MAX, nsec;
 
 	pw_log_trace_fp("%p: ready driver:%d exported:%d %p status:%d prepared:%d", node,
@@ -2096,20 +2109,6 @@ static int node_ready(void *data, int status)
 		}
 	}
 
-	/* This update is done too late, the driver should do this
-	 * before calling the ready callback so that it can use the new target
-	 * duration and rate to schedule the next update. We do this here to
-	 * help drivers that don't support this yet */
-	if (SPA_UNLIKELY(cl->duration != cl->target_duration ||
-	    cl->rate.denom != cl->target_rate.denom)) {
-		pw_log_warn("driver %s did not update duration/rate (%"PRIu64"/%"PRIu64" %u/%u)",
-				node->name,
-				cl->duration, cl->target_duration,
-				cl->rate.denom, cl->target_rate.denom);
-		cl->duration = cl->target_duration;
-		cl->rate = cl->target_rate;
-	}
-
 	sync_type = check_updates(node, &reposition_owner);
 	owner[0] = SPA_ATOMIC_LOAD(a->segment_owner[0]);
 	owner[1] = SPA_ATOMIC_LOAD(a->segment_owner[1]);
@@ -2117,6 +2116,7 @@ again:
 	all_ready = sync_type == SYNC_CHECK;
 	update_sync = !all_ready;
 	target_sync = sync_type == SYNC_START ? true : false;
+	pending = 0;
 
 	spa_list_for_each(t, &driver->rt.target_list, link) {
 		struct pw_node_activation *ta = t->activation;
@@ -2125,6 +2125,14 @@ again:
 		ta->driver_id = driver->info.id;
 retry_status:
 		pw_node_activation_state_reset(&ta->state[0]);
+
+		if (ta->active_driver_id != ta->driver_id) {
+			pw_log_trace_fp("%p: (%s-%u) %d waiting for driver %d<>%d", t->node,
+					t->name, t->id, ta->status,
+					ta->active_driver_id, ta->driver_id);
+			continue;
+		}
+
 		/* we don't change the state of inactive nodes and don't use them
 		 * for reposition. The pending will be at least 1 and they might
 		 * get decremented to 0 but since the status is inactive, we don't
@@ -2138,6 +2146,9 @@ retry_status:
 		if (SPA_UNLIKELY(!SPA_ATOMIC_CAS(ta->status, old_status, PW_NODE_ACTIVATION_NOT_TRIGGERED)))
 			goto retry_status;
 
+		if (!SPA_FLAG_IS_SET(ta->flags, PW_NODE_ACTIVATION_FLAG_ASYNC))
+			pending++;
+
 		if (old_status == PW_NODE_ACTIVATION_TRIGGERED ||
 		    old_status == PW_NODE_ACTIVATION_AWAKE) {
 			update_xrun_stats(ta, 1, nsec / 1000, 0);
@@ -2178,6 +2189,7 @@ retry_status:
 		reposition_target = NULL;
 		goto again;
 	}
+	state->pending = pending;
 
 	update_position(node, all_ready, nsec);
 
@@ -2450,6 +2462,7 @@ void pw_impl_node_destroy(struct pw_impl_node *node)
 	pw_work_queue_cancel(impl->work, node, SPA_ID_INVALID);
 
 	pw_properties_free(node->properties);
+	spa_clear_ptr(impl->pending_request_process, free);
 
 	clear_info(node);
 
@@ -2846,7 +2859,7 @@ int pw_impl_node_send_command(struct pw_impl_node *node, const struct spa_comman
 
 	switch (id) {
 	case SPA_NODE_COMMAND_RequestProcess:
-		handle_request_process(node);
+		handle_request_process_command(node, command);
 		break;
 	default:
 		res = spa_node_send_command(node->node, command);
